@@ -1,5 +1,5 @@
-/* Copyright (C) 2012-2013, 2015-2022, 2024 D. R. Commander.
- *                                          All Rights Reserved.
+/* Copyright (C) 2012-2013, 2015-2022, 2024-2025 D. R. Commander.
+ *                                               All Rights Reserved.
  * Copyright (C) 2011-2013 Brian P. Hinz
  * Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
  *
@@ -130,8 +130,12 @@ public final class Viewport extends JFrame implements Runnable {
               (availableSize.width != cc.desktop.scaledWidth ||
                availableSize.height != cc.desktop.scaledHeight)) {
             sp.setHorizontalScrollBarPolicy(
+              cc.params.fullScreen.get() && cc.params.bumpScroll.get() ?
+              ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER :
               ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
             sp.setVerticalScrollBarPolicy(
+              cc.params.fullScreen.get() && cc.params.bumpScroll.get() ?
+              ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER :
               ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
             sp.validate();
             if (timer != null)
@@ -151,8 +155,12 @@ public final class Viewport extends JFrame implements Runnable {
           }
         } else {
           sp.setHorizontalScrollBarPolicy(
+            cc.params.fullScreen.get() && cc.params.bumpScroll.get() ?
+            ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER :
             ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
           sp.setVerticalScrollBarPolicy(
+            cc.params.fullScreen.get() && cc.params.bumpScroll.get() ?
+            ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER :
             ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
           sp.validate();
         }
@@ -271,7 +279,7 @@ public final class Viewport extends JFrame implements Runnable {
     Dimension borderSize = new Dimension(vpInsets.left + vpInsets.right,
                                          vpInsets.top + vpInsets.bottom);
     if (cc.params.toolbar.get())
-      borderSize.height += 22;
+      borderSize.height += 26;
     return borderSize;
   }
 
@@ -452,6 +460,23 @@ public final class Viewport extends JFrame implements Runnable {
       cc.primaryGD.setFullScreenWindow(on ? this : null);
   }
 
+  static final int NS_APPLICATION_PRESENTATION_HIDE_DOCK = (1 << 1);
+  static final int NS_APPLICATION_PRESENTATION_HIDE_MENU_BAR = (1 << 3);
+
+  private native void macFullScreen(boolean on, int presentationOptions);
+
+  public void macFullScreenHelper(boolean on) {
+    if (!Helper.isAvailable()) return;
+    int presentationOptions = 0;
+    boolean hide;
+    if (Utils.getBooleanProperty("turbovnc.fshidedock",
+                                 cc.params.bumpScroll.get()))
+      presentationOptions |= NS_APPLICATION_PRESENTATION_HIDE_DOCK |
+                             NS_APPLICATION_PRESENTATION_HIDE_MENU_BAR;
+    if (presentationOptions != 0)
+      macFullScreen(on, presentationOptions);
+  }
+
   public void grabKeyboardHelper(boolean on) {
     grabKeyboardHelper(on, false);
   }
